import gui
import pygame
from quit import *
from game import *
from gui.dialog import *
from gui.image import *

class EndScreen(Dialog):
        def __init__(self, surface, images, locations, player):
                self.images = images
                self.locations = locations
		self.player = player
		Dialog.__init__(self, surface)
		
	def createWidgets(self):

		self.wm.register(Image(self.images['endscreen']['winner'+str(self.player)],
			(int(self.locations['endscreen']['winner_x']),
			int(self.locations['endscreen']['winner_y']))
		))

		self.wm.register(Button(self.images['endscreen']['return'], self.images['endscreen']['return'],
			(int(self.locations['endscreen']['return_x']),
			int(self.locations['endscreen']['return_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.return_to_main }
                ))

        def return_to_main(self, trigger, event):
		self.state = 1
		return widget.DONE



import pygame
from quit import *
from game import *
from gamescreen import *
from settingsscreen import *
from gui.dialog import *
from gui.image import *
from players import *
from creditsscreen import *

class StartScreen(Dialog):
	def __init__(self, surface, images, locations, ini_settings):
		self.images = images
		self.locations = locations
		self.ini_settings = ini_settings	
		Dialog.__init__ (self, surface)
		
	def createWidgets(self):
		# Set the background
		self.surface.blit(self.images['startscreen']['background'],(0,0))

		self.wm.register(Image(self.images['startscreen']['title'],
			(int(self.locations['startscreen']['title_x']),
			int(self.locations['startscreen']['title_y']))
		))
		self.wm.register(Button(self.images['startscreen']['newgame'], self.images['startscreen']['newgame'],
			(int(self.locations['startscreen']['newgame_x']),
			int(self.locations['startscreen']['newgame_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.newgame }
		))

		
		self.wm.register(Button(self.images['startscreen']['credits'], self.images['startscreen']['credits'],
			(int(self.locations['startscreen']['credits_x']),
			int(self.locations['startscreen']['credits_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.credits }
		))
		
		
		self.wm.register(Button(self.images['startscreen']['settings'], self.images['startscreen']['settings'],
			(int(self.locations['startscreen']['settings_x']),
			int(self.locations['startscreen']['settings_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.settings }
		))

		self.wm.register(Button(self.images['startscreen']['quit'], self.images['startscreen']['quit'],
			(int(self.locations['startscreen']['quit_x']),
			int(self.locations['startscreen']['quit_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.quit }
		))

	def newgame(self, trigger, event):
		gamescreen = GameScreen(self.surface, self.images, self.locations)
		gamescreen.run()
		self.surface.blit(self.images['startscreen']['background'],(0,0))
		self.wm.paint(1,0)

	def settings(self, trigger, event):
		settingsscreen = SettingsScreen(self.surface, self.images, self.locations, self.ini_settings)
		settingsscreen.run()
                self.surface.blit(self.images['startscreen']['background'],(0,0))
                self.wm.paint(1,0)

	def credits(self, trigger, event):
		creditsscreen = CreditsScreen(self.surface, self.images, self.locations)
		creditsscreen.run()
                self.surface.blit(self.images['startscreen']['background'],(0,0))
                self.wm.paint(1,0)

	def quit(self, trigger, event):
		quit()

	def loadPlayer(self, name):
                difficulty = 2 #int(self.settings.getGlobal()['game'][name+'_level'])

                if self.settings.getGlobal()['game'][name+'_type'] == 'human':
	                player = Human(self.screen, self.images, self.settings, difficulty)
                elif self.settings.getGlobal()['game'][name+'_type'] == 'strategic':
                        player = StrategicMinMax(self.screen, self.images, self.settings, difficulty)
		elif self.settings.getGlobal()['game'][name+'_type'] == 'random':
                        player = RandomAI(self.screen, self.images, self.settings, difficulty)
                elif self.settings.getGlobal()['game'][name+'_type'] == 'minmax':
			player = MinMax(self.screen, self.images, self.settings, difficulty)
                elif self.settings.getGlobal()['game'][name+'_type'] == 'weighted':
                        player = WeightedMinMax(self.screen, self.images, self.settings, difficulty)
                else:
			print "INI FILE CURRUPT: set a player type to a legal one"
		        print "Legal players include:"
		        print "human, strategic, random, minmax"
		        quit()

		return player



###############################################################################################
# $Id: start_game.py,v 1.1.1.1 2002/02/19 10:16:04 slmm Exp $
###############################################################################################

import pygame
from pygame.locals import *
import scenario
import properties
import traceback
import sys
import string
import widget
from button          import Button
from label           import Label
from image           import Image
from widget_manager  import WidgetManager
from dialog          import *
from messagebox      import Messagebox
from scenario_parser import ScenarioParser

class StartGame(Dialog):
    """
    This class is used as a dialog for showing some information to the player while the game sets up
    the network connection and receives the scenario information. Shows a centered splash screen
    with a label of information on it.
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )
        
        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # load the images for the progress bar
        #self.progressimages = []
        #for image in (properties.progress_bar_start, properties.progress_bar_mid, properties.progress_bar_end):
            

        # no error yet
        self.error = ()

        # no lines to read yet
        self.linecount = -1

        # neither scenario data not config data read
        self.configread = 0
        self.scenarioread = 0
        
        # no scenario data yet
        self.buffer = ''
        
        # we want timer events
        self.enableTimer ( 300 )
        

    def createWidgets (self):
        "Creates all widgets for the dialog."

        # create a message
        message = "Waiting for other player to perform setup..."
            
        # create the splash
        self.splash = Image ( properties.dialog_splash, (272, 204) )
        self.wm.register ( self.splash )
 
        # the status label
        self.statuslabel = Label (self.smallfont, message, (340, 213), color=(255, 255, 255))
        self.wm.register ( self.statuslabel )

        # create the cancel button
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (406, 650), {widget.MOUSEBUTTONUP : self.cancel } ) )



    def timer (self):
        """Callback triggered when the dialog has enabled timers and a timer fires. First tries to
        read the configuration from the connection, and if that succeeds further calls will read the
        raw scenario data. If all succeeds the dialog is accepted."""

        # do we have lines to read, i.e. have we started to receive the scenario?
        if not self.configread:
            # read scenario config
            return self.readConfig ()

        elif not self.scenarioread:
            # read the scenario data instead
            return self.readScenario ()

        # we're cancelling the dialog
        self.state = ACCEPTED
        return widget.DONE


    def readConfig (self):
        """Reads the configuration data from the network connection. If all is ok sets the flag that
        indicates that the configuration is now read."""
        
        # wait for the server to send a status
        status = scenario.connection.readLine ( timeout = 0.01 )

        # did we get a status
        if not status:
            # nothing was read, go on
            return

        # split up the received message into components
        parts = status.split ()

        # precautions. we need at least 4 parts: "start id lines name"
        if len ( parts ) < 4 or parts[0] != 'start':
            # oops, bad data
            Messagebox ( "Bad data received from server!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
 
            # we're cancelling the dialog
            self.state = REJECTED
        
            return widget.DONE

        # get all data from the read parts
        scenario.local_player_id = int ( parts[1] )
        scenario.remote_player_name = string.join ( parts [3:] )

        # also the number of lines in the scenario we're about to read
        self.linecount = int ( parts[2] )

        # configuration is now read
        self.configread = 1

        return widget.HANDLED

        
    def readScenario (self):
        """Retrieves a scenario from the server. Sends the id of the scenario to the server and
        reads the XML data the server sends back."""

        # optimize away a dot
        connection = scenario.connection
        
        try:
            lastratio = 0
            xpos = 352
            ypos = 461
            
            # create initial progress icon, the leftmost
##             progress = Image ( properties.progress_bar_start, ( 100, 200 ) )
##             self.wm.register ( progress )

            # let the player know what we do
            self.statuslabel.setText ( 'Downloading scenario...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # loop and read the lines
            for index in range ( self.linecount ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                self.buffer += line

                # get a ratio [0..9] of the progress
                ratio = int ( float (index) / self.linecount * 33.0 )

                # have we advanced another 10%?
                if ratio > lastratio:
                    # yep, let the user feel the update
                    lastratio = ratio

                    # create a new part of the progress bar
                    progress = Image ( filename=properties.progress_bar_mid, position=(xpos, ypos))
                    self.wm.register ( progress )

                    # go onwards
                    xpos += 10
                    
                    # we must repaint the entire dialog
                    self.wm.paint ( clear=0, force=0 )

            # let the player know what we do
            self.statuslabel.setText ( 'Building scenario...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # parse the read data
            ScenarioParser ().parseString ( self.buffer )

            # scenario is read now
            self.scenarioread = 1
           
            # we're done with the dialog
            return widget.HANDLED
        
        except:
            raise
            # serious error
            Messagebox ( "Could not read scenario!" )
            
            # repaint and go away
            self.wm.paint (force=1, clear=1)
 
            # we're cancelling the dialog
            self.state = REJECTED
            return widget.DONE


    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""

        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



from player import *
import gui
import pygame, pygame.font, pygame.image, pygame.mixer, pygame.transform
from pygame.locals import *
import rules
from gui.dialog import *

class Human(Dialog):
	type = 'human'

	def __init__(self, surface, images, locations, difficulty):
		self.images = images
		self.locations = locations
		Dialog.__init__ (self, surface)

	def createWidgets(self):
		for number in range(7):
			self.wm.register(Button(self.images['human']['column'+ str(number)], self.images['human']['column'+ str(number)],
	                        (int(self.locations['game']['column'+ str(number) +'_x']),
	                        int(self.locations['game']['column'+ str(number) +'_y'])),
	                        callbacks={widget.MOUSEBUTTONUP : self.setmove }, args=number
	                ))

	# This method returns the move to be made
	def doMove(self, board, player, event):
		self.move = -1
		self.board = board
		# Reset the move to an illegal one
		self.move = -1
		self.handleEvent(event)
		if self.move >-1 and self.move < 7:
			return self.move
		return -1

	def setmove(self, trigger, event, number):
		if rules.isMoveLegal(self.board, number):
			self.move = number
			return widget.DONE
	def gameOver(self, move):
		return None



import time

class GameLogger:
	def newgame(self):
		self.move_number = 0
		self.logfile = open( "gamelog-%s.txt" % time.clock(), 'w')

	def logmove(self, move):
		self.move_number += 1
		self.logfile.write("%s,	%s\n" % (self.move_number, move))



###############################################################################################
# $Id: wait_client.py,v 1.1.1.1 2002/02/19 10:16:04 slmm Exp $
###############################################################################################

import pygame
from pygame.locals import *
import scenario
import properties
import traceback
import sys
import string
import widget
from net.connection  import Connection
from select          import select
from button          import Button
from label           import Label
from image           import Image
from widget_manager  import WidgetManager
from dialog          import *
from messagebox      import Messagebox
from scenario_parser import ScenarioParser

class WaitClient(Dialog):
    """
    This class is used as a dialog for showing 
    """

    def __init__ (self, sock):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # store the socket
        self.socket = sock

        # we want timer events
        self.enableTimer ( 300 )
        

    def createWidgets (self):
        "Creates all widgets for the dialog."

        # create a message
        message = "Waiting for client to connect..."
 
        # the status label
        self.label = Label (self.smallfont, message, (340, 213), color=(255, 255, 255))
        self.wm.register ( self.label )

        # create the cancel button
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (406, 650), {widget.MOUSEBUTTONUP : self.cancel } ) )



    def timer (self):
        """Callback triggered when the dialog has enabled timers and a timer fires. First tries to
        read the configuration from the connection, and if that succeeds further calls will read the
        raw scenario data. If all succeeds the dialog is accepted."""

        # perform the select. We're only interested in incoming events. wait forever
        incoming, out, execptional = select ( [self.socket], [], [], 0.0 )

        # was there something on the incoming socket that we listen on?
        if self.socket in incoming:
            # yep, we need to accept the new client
 
            # wait and accept the new client
            newsock, addr = self.socket.accept ()

            # create and return the new sockets as connections
            scenario.connection = Connection ( socket=newsock, host="localhost" )

            # we're done the dialog
            self.state = ACCEPTED
            return widget.DONE


    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""

        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



from board import *
from players import *
from rules import *
from endscreen import *
import pygame, pygame.font, pygame.image, pygame.mixer, pygame.transform
from pygame.locals import *
import gui
from quit import *
from gamelog import *
import types
from endgamedialog import *

class Game:
	def __init__(self, screen, images, locations, player1, player2):
		self.screen = screen
		self.images = images
		self.locations = locations
		self.player1 = player1
		self.player2 = player2
			
	def run(self):
		self.gameloop()	
	
	# Graphics methods
	def drawPiece(self, piece, x, y):
        	self.screen.blit( piece, (
			int(self.locations['board']['x']) + x * piece.get_width(),
			int(self.locations['board']['y']) + (5-y) * piece.get_height()))
	
	def drawPieces(self, board):
        	for x in range(7):
                	for y in range(len(board.state[x])):
                        	if   board.state[x][y] == 1:
					self.drawPiece(self.images['game']['stone1'], x, y)
	                        elif board.state[x][y] == 2:
					self.drawPiece(self.images['game']['stone2'], x, y)

	def drawBoard(self, board):
        	self.screen.blit(self.images['game']['back'], (
			int(self.locations['board']['x']),
			int(self.locations['board']['y'])))

	        self.drawPieces(board)

        	self.drawGrid()
	        return self.screen

	def drawGrid(self):
        	self.screen.blit(self.images['game']['grid'], (
        		int(self.locations['board']['x']),
                        int(self.locations['board']['y'])))

	def doMove(self, selector_pos, stone):
		board.state[selector_pos].append(stone)
	        return board
	
	# Display the winner screen
	def winner(self, player):
		endscreen = EndScreen(self.screen, self.images, self.locations, player)
	        endscreen.run()
	        return


	# The gameloop
	def gameloop(self):
		self.screen.blit(self.images['game']['background'], (0,0))

		player1 = self.player1
		player2 = self.player2

        	# Board generating code
	        board = Board()
        	# Board generating code

	        self.drawBoard(board)
        	pygame.display.flip()

		#Can be used for logging a game
		#gamelogger = GameLogger()
		#gamelogger.newgame()

	        while 1:
			while 1:
    				event = pygame.event.poll()

				#self.chat.handleEvent(event)

				if event.type is KEYDOWN and event.key is K_ESCAPE:
					quitdialog = EndGameDialog(self.screen, self.images, self.locations)
					if quitdialog.run() is "quit":
						quit()
					else:
						self.drawBoard(board)
						pygame.display.flip()


        	        	move = player1.doMove(board, 1, event)
				if type(move) is types.IntType and rules.isMoveLegal(board, move):
					board.move(move, 1)
					break

			self.drawBoard(board)
			pygame.display.flip()

        	        if rules.isWinner(board, 1):
				player2.gameOver(move)
				player1.gameOver(move)
				return self.winner(1)
			elif rules.isBoardFull(board):
				player2.gameOver(move)
				player1.gameOver(move)
				return self.winner(0)

			while 1:
				event = pygame.event.poll()
				#self.chat.handleEvent(event)

        			if event.type is KEYDOWN and event.key is K_ESCAPE:
					quitdialog = EndGameDialog(self.screen, self.images, self.locations)
					if quitdialog.run() is "quit":
						quit()
					else:
						self.drawBoard(board)
						pygame.display.flip()


				move = player2.doMove(board, 2, event)
				if type(move) is types.IntType and rules.isMoveLegal(board, move):
                                	board.move(move, 2)
					break

			self.drawBoard(board)
	                pygame.display.flip()

			if rules.isWinner(board, 2):
				player1.gameOver(move)
				player2.gameOver(move)
				return self.winner(2)
			elif rules.isBoardFull(board):
				player1.gameOver(move)
				player2.gameOver(move)
				return self.winner(0)
